#FILE: build_cluster_script.pl
#AUTH: Jason Grant (jasonrgrant@gmail.com)
#AUTH: Paul Stothard (stothard@ualberta.ca)
#DATE: Nov. 15, 2007
#VERS: 1.1

use warnings;
use strict;
use Getopt::Long;

#initialize options
my %options = (
    command  => undef,
    outfile  => undef,
    number   => undef,
    letters  => undef,
    separate => undef
);

#get command line options
GetOptions(
    'c=s' => \$options{command},
    'o=s' => \$options{outfile},
    'n=i' => \$options{number},
    'l'   => \$options{letters},
    's'   => \$options{separate}
);

#check for options
if (   !( defined( $options{command} ) )
    or !( defined( $options{outfile} ) )
    or !( defined( $options{number} ) ) )
{
    die( print_usage() . "\n" );
}

my @commands = ();

#write out the command with '$' replaced by the counter
if ( !defined( $options{letters} ) ) {
    for my $counter ( 1 .. $options{number} ) {
        my $temp_command = $options{command};
        $temp_command =~ s/\$/$counter/g;
        push( @commands,
            { command => $temp_command, increment => $counter } );
    }
}
else {

    #Add an extension of letters, eg _part_aa, _part_ab, etc. $levels
    #determines how many letters are added.
    my @abc    = ( 'a' .. 'z' );
    my $levels = 2;
    if ( $options{number} > scalar(@abc)**$levels ) {
        die("ERROR: Unable to represent file number $options{number} using $levels letters."
        );
    }
    for my $counter ( 1 .. $options{number} ) {
        my $temp_command = $options{command};
        my $replacement
            = get_letter_extension( \@abc, $levels, $counter, "" );
        $temp_command =~ s/\$/$replacement/g;
        push( @commands,
            { command => $temp_command, increment => $replacement } );
    }
}

if ( !defined( $options{separate} ) ) {
    open( my $OUTFILE, '>', "$options{outfile}" )
        or die("Cannot open file '$options{outfile}': $!");
    print {$OUTFILE} "#!/bin/bash\n";
    foreach (@commands) {
        print {$OUTFILE} $_->{command} . "\n";
    }
    close($OUTFILE) or die("Cannot close file '$options{outfile}': $!");
    chmod( 0744, $options{outfile} );
}
else {
    foreach (@commands) {
        my $infile_base      = "";
        my $infile_extension = "";
        if ( $options{outfile} =~ /(.+)(\..+?)$/ ) {
            $infile_base      = $1;
            $infile_extension = $2;
        }
        my $file = $infile_base . "_" . $_->{increment} . $infile_extension;
        open( my $OUTFILE, '>', "$file" )
            or die("Cannot open file '$file': $!");
        #2011-04-18
        #print {$OUTFILE} "#!/bin/bash\n";
        print {$OUTFILE} "#\$ -S /bin/bash\n";
        print {$OUTFILE} $_->{command} . "\n";
        close($OUTFILE) or die("Cannot close file '$file': $!");
        chmod( 0744, $file );
    }
}

sub get_letter_extension {
    my ( $abc, $level, $number, $extension ) = @_;
    if ( $level == 0 ) {
        return $extension;
    }
    if ( $level > 1 ) {
        my $div = int( ( $number - 1 ) / scalar(@$abc) );
        if ( $div >= scalar(@$abc) ) {
            $div = scalar(@$abc) - 1;
        }
        $extension = $extension . $abc->[$div];
        $number    = $number - $div * scalar(@$abc);
    }
    else {
        my $mod = $number % scalar(@$abc);
        $extension = $extension . $abc->[ $mod - 1 ];
        $number    = $number - $mod;
    }
    $level--;
    return get_letter_extension( $abc, $level, $number, $extension );
}

sub print_usage {
    print <<BLOCK;
USAGE: perl build_cluster_script.pl [-arguments]
-c [STRING]   : command to be repeated, '$' will be replaced by
                increasing number (Required).
-o [FILE]     : output file (Required).
-n [INTEGER]  : number of commands to create (Required).
-l            : use letters instead of numbers when creating commands.
-s            : write each command to a separate script file.

perl build_cluster_script.pl -c 'qsub submit.sh -i input$.fa \
-o output$.fa' -o myscript.sh -n 20
BLOCK
}
